@@ -17,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmarks.c 8149 2008-03-28 15:07:39Z chpe $
  */
 
 #include "config.h"
@@ -38,14 +37,12 @@
 #include "ephy-marshal.h"
 #include "ephy-signal-accumulator.h"
 #include "ephy-stock-icons.h"
-#include "ephy-glib-compat.h"
 
 #include "eel-gconf-extensions.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #ifdef ENABLE_ZEROCONF
 #include <avahi-common/error.h>
@@ -96,21 +93,6 @@ struct _EphyBookmarksPrivate
 #endif
 };
 
-typedef struct
-{
-	const char *title;
-	const char *location;
-} EphyBookmarksBookmarkInfo;
-
-static const EphyBookmarksBookmarkInfo default_bookmarks [] =
-{
-	/* Translators you should change these links to respect your locale.
-	 * For instance in .nl these should be
-	 * "http://www.google.nl" and "http://www.google.nl/search?q=%s"
-	 */
-	{ N_("Search the web"), N_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8") }
-};
-
 static const char *default_topics [] =
 {
 	N_("Entertainment"),
@@ -129,12 +111,15 @@ enum
 	LAST_SIGNAL
 };
 
-static guint ephy_bookmarks_signals[LAST_SIGNAL] = { 0 };
+static guint ephy_bookmarks_signals[LAST_SIGNAL];
 
 static void ephy_bookmarks_class_init	(EphyBookmarksClass *klass);
 static void ephy_bookmarks_init		(EphyBookmarks *tab);
 static void ephy_bookmarks_finalize	(GObject *object);
 static char *impl_resolve_address	(EphyBookmarks*, const char*, const char*);
+#ifdef ENABLE_ZEROCONF
+static void ephy_local_bookmarks_start_client (EphyBookmarks *bookmarks);
+#endif
 
 G_DEFINE_TYPE_WITH_CODE (EphyBookmarks, ephy_bookmarks, G_TYPE_OBJECT,
 			 volatile GType flags_type; /* work around gcc's optimiser */
@@ -151,13 +136,7 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 		ephy_bookmarks_add_keyword (eb, _(default_topics[i]));
 	}
 
-	for (i = 0; i < G_N_ELEMENTS (default_bookmarks); i++)
-	{
-		EphyNode *bmk;
-
-		bmk = ephy_bookmarks_add (eb, _(default_bookmarks[i].title),
-					  _(default_bookmarks[i].location));
-	}
+	ephy_bookmarks_import_rdf (eb, DATADIR "/default-bookmarks.rdf");
 }
 
 static void
@@ -960,7 +939,10 @@ resolver_found_cb (GaServiceResolver *resolver,
 	
 	ephy_node_db_set_immutable (priv->db, was_immutable);
 
-	g_hash_table_unref (text_table);
+	if (text_table != NULL)
+	{
+		g_hash_table_unref (text_table);
+	}
 }
 
 static void
@@ -1133,14 +1115,12 @@ ga_client_state_changed_cb (GaClient *ga_client,
 	{
 		if (avahi_client_errno (ga_client->avahi_client) == AVAHI_ERR_DISCONNECTED)
 		{
-			/* FIXMEchpe: is this correct */
-			/* Destroy and reconnect */
-			avahi_client_free (ga_client->avahi_client);
-			ga_client->avahi_client = NULL;
-			if (!ga_client_start (ga_client, NULL))
-			{
-				g_warning ("Unable to start Zeroconf subsystem");
-			}
+                        EphyBookmarksPrivate *priv = bookmarks->priv;
+
+                        g_object_unref (priv->ga_client);
+                        priv->ga_client = NULL;
+
+                        ephy_local_bookmarks_start_client (bookmarks);
 		}
 	}
 	if (state == GA_CLIENT_STATE_S_RUNNING)
@@ -1150,7 +1130,7 @@ ga_client_state_changed_cb (GaClient *ga_client,
 }
 
 static void
-ephy_local_bookmarks_init (EphyBookmarks *bookmarks)
+ephy_local_bookmarks_start_client (EphyBookmarks *bookmarks)
 {
 	EphyBookmarksPrivate *priv = bookmarks->priv;
 	GaClient *ga_client;
@@ -1162,12 +1142,20 @@ ephy_local_bookmarks_init (EphyBookmarks *bookmarks)
 	if (!ga_client_start (ga_client, NULL))
 	{
 		g_warning ("Unable to start Zeroconf subsystem");
+                g_object_unref (ga_client);
 		return;
 	}
 	priv->ga_client = ga_client;
+}
+
+static void
+ephy_local_bookmarks_init (EphyBookmarks *bookmarks)
+{
+	EphyBookmarksPrivate *priv = bookmarks->priv;
 	priv->resolve_handles =	g_hash_table_new_full (g_str_hash, g_str_equal,
 						       g_free,
 						       (GDestroyNotify) resolve_data_free);
+        ephy_local_bookmarks_start_client (bookmarks);
 }
 
 static void
@@ -1211,32 +1199,20 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 {
 	EphyNodeDb *db;
 
-	/* Translators: The text before the "|" is context to help you
-	 * decide on the correct translation. You MUST OMIT it in the
-	 * translated string. */
 	/* Translators: this topic contains all bookmarks */
-	const char *bk_all = Q_("bookmarks|All");
+	const char *bk_all = C_("bookmarks", "All");
 
-	/* Translators: The text before the "|" is context to help you
-	 * decide on the correct translation. You MUST OMIT it in the
-	 * translated string. */
 	/* Translators: this topic contains the most used bookmarks */
-	const char *bk_most_visited = Q_("bookmarks|Most Visited");
+	const char *bk_most_visited = C_("bookmarks", "Most Visited");
 
-	/* Translators: The text before the "|" is context to help you
-	 * decide on the correct translation. You MUST OMIT it in the
-	 * translated string. */
 	/* Translators: this topic contains the not categorized
 	   bookmarks */
-	const char *bk_not_categorized = Q_("bookmarks|Not Categorized");
+	const char *bk_not_categorized = C_("bookmarks", "Not Categorized");
 	
-#ifdef ENABLE_ZEROCONF	
-	/* Translators: The text before the "|" is context to help you
-	 * decide on the correct translation. You MUST OMIT it in the
-	 * translated string. */
+#ifdef ENABLE_ZEROCONF
 	/* Translators: this is an automatic topic containing local
 	 * websites bookmarks autodiscovered with zeroconf. */
-	const char *bk_local_sites = Q_("bookmarks|Nearby Sites");
+	const char *bk_local_sites = C_("bookmarks", "Nearby Sites");
 #endif
 
 	eb->priv = EPHY_BOOKMARKS_GET_PRIVATE (eb);
@@ -1961,30 +1937,55 @@ ephy_bookmarks_unset_keyword (EphyBookmarks *eb,
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
+/**
+ * ephy_bookmarks_get_smart_bookmarks:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_smart_bookmarks (EphyBookmarks *eb)
 {
 	return eb->priv->smartbookmarks;
 }
 
+/**
+ * ephy_bookmarks_get_keywords:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_keywords (EphyBookmarks *eb)
 {
 	return eb->priv->keywords;
 }
 
+/**
+ * ephy_bookmarks_get_bookmarks:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_bookmarks (EphyBookmarks *eb)
 {
 	return eb->priv->bookmarks;
 }
 
+/**
+ * ephy_bookmarks_get_favorites:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_favorites (EphyBookmarks *eb)
 {
 	return eb->priv->favorites;
 }
 
+/**
+ * ephy_bookmarks_get_local:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_local (EphyBookmarks *eb)
 {
@@ -1995,12 +1996,22 @@ ephy_bookmarks_get_local (EphyBookmarks *eb)
 #endif
 }
 
+/**
+ * ephy_bookmarks_get_not_categorized:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_not_categorized (EphyBookmarks *eb)
 {
 	return eb->priv->notcategorized;
 }
 
+/**
+ * ephy_bookmarks_get_from_id:
+ *
+ * Return value: (transfer none):
+ **/
 EphyNode *
 ephy_bookmarks_get_from_id (EphyBookmarks *eb, long id)
 {
