@@ -14,40 +14,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-history-window.c,v 1.106.2.3 2006/10/02 20:29:35 chpe Exp $
  */
 
 #include "config.h"
 
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkradioaction.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkhpaned.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkclipboard.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkuimanager.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
 #include <time.h>
 
-#include "ephy-node-view.h"
 #include "ephy-window.h"
 #include "ephy-history-window.h"
 #include "ephy-shell.h"
@@ -67,7 +44,9 @@
 #include "ephy-node-common.h"
 #include "ephy-node-view.h"
 #include "ephy-bookmarks-ui.h"
+#include "ephy-prefs.h"
 #include "ephy-gui.h"
+#include "ephy-time-helpers.h"
 
 static const GtkTargetEntry page_drag_types [] =
 {
@@ -111,6 +90,11 @@ static void cmd_select_all		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_help_contents		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
+static void cmd_view_columns		  (GtkAction *action,
+					   EphyHistoryWindow *view);
+static void search_entry_search_cb 	  (GtkWidget *entry,
+					   char *search_text,
+					   EphyHistoryWindow *editor);
 
 #define EPHY_HISTORY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_HISTORY_WINDOW, EphyHistoryWindowPrivate))
 
@@ -134,6 +118,7 @@ struct _EphyHistoryWindowPrivate
 	EphyNode *selected_site;
 	GtkTreeViewColumn *title_col;
 	GtkTreeViewColumn *address_col;
+	GtkTreeViewColumn *datetime_col;
 };
 
 enum
@@ -157,8 +142,6 @@ enum
 #define TIME_LAST_TWO_DAYS_STRING "last_two_days"
 #define TIME_LAST_THREE_DAYS_STRING "last_three_days"
 
-static GObjectClass *parent_class = NULL;
-
 static const GtkActionEntry ephy_history_ui_entries [] = {
 	/* Toplevel */
 	{ "File", NULL, N_("_File") },
@@ -174,10 +157,7 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "OpenInTab", STOCK_NEW_TAB, N_("Open in New _Tab"), "<shift><control>O",
 	  N_("Open the selected history link in a new tab"),
 	  G_CALLBACK (cmd_open_bookmarks_in_tabs) },
-	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
-	  N_("Delete the selected history link"),
-	  G_CALLBACK (cmd_delete) },
-	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("Add _Bookmark..."), "<control>D",
+	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("Add _Bookmark…"), "<control>D",
 	  N_("Bookmark the selected history link"),
 	  G_CALLBACK (cmd_bookmark_link) },
 	{ "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -194,7 +174,10 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 	  N_("Paste the clipboard"),
 	  G_CALLBACK (cmd_paste) },
-	{ "SelectAll", NULL, N_("Select _All"), "<control>A",
+	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), "<control>T",
+	  N_("Delete the selected history link"),
+	  G_CALLBACK (cmd_delete) },
+	{ "SelectAll", GTK_STOCK_SELECT_ALL, N_("Select _All"), "<control>A",
 	  N_("Select all history links or text"),
 	  G_CALLBACK (cmd_select_all) },
 	{ "Clear", GTK_STOCK_CLEAR, N_("Clear _History"), NULL,
@@ -205,28 +188,27 @@ static const GtkActionEntry ephy_history_ui_entries [] = {
 	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
 	  N_("Display history help"),
 	  G_CALLBACK (cmd_help_contents) },
-	{ "HelpAbout", GNOME_STOCK_ABOUT, N_("_About"), NULL,
+	{ "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
 	  N_("Display credits for the web browser creators"),
 	  G_CALLBACK (window_cmd_help_about) },
 };
 
-enum
+typedef enum
 {
-	VIEW_TITLE,
-	VIEW_ADDRESS,
-	VIEW_TITLE_AND_ADDRESS
-};
+	VIEW_TITLE = 1 << 0,
+	VIEW_ADDRESS = 1 << 1,
+	VIEW_DATETIME = 1 << 2
+} EphyHistoryWindowColumns;
 
-static const GtkRadioActionEntry ephy_history_radio_entries [] =
+static const GtkToggleActionEntry ephy_history_toggle_entries [] =
 {
 	/* View Menu */
 	{ "ViewTitle", NULL, N_("_Title"), NULL,
-	  N_("Show only the title column"), VIEW_TITLE },
+	  N_("Show the title column"), G_CALLBACK (cmd_view_columns), TRUE },
 	{ "ViewAddress", NULL, N_("_Address"), NULL,
-	  N_("Show only the address column"), VIEW_ADDRESS },
-	{ "ViewTitleAddress", NULL, N_("Title a_nd Address"), NULL,
-	  N_("Show both the title and address columns"),
-	  VIEW_TITLE_AND_ADDRESS } 
+	  N_("Show the address column"), G_CALLBACK (cmd_view_columns), TRUE },
+	{ "ViewDateTime", NULL, N_("_Date and Time"), NULL,
+	  N_("Show the date and time column"), G_CALLBACK (cmd_view_columns), TRUE }
 };
 
 static void
@@ -262,7 +244,7 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (editor)),
 				     GTK_WINDOW (dialog));
 	
-	button = gtk_button_new_with_label (_("Cl_ear"));
+	button = gtk_button_new_with_mnemonic (_("Cl_ear"));
 	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image (GTK_BUTTON (button), image);
 	/* don't show the image! see bug #307818 */
@@ -272,7 +254,7 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Clear History"));
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (confirmation_dialog_response_cb),
@@ -475,6 +457,8 @@ cmd_delete (GtkAction *action,
 		{
 			ephy_node_view_remove (EPHY_NODE_VIEW (editor->priv->sites_view));
 		}
+		
+		g_list_free (selected);
 	}
 }
 
@@ -506,84 +490,81 @@ static void
 cmd_help_contents (GtkAction *action,
 		   EphyHistoryWindow *editor)
 {
-	ephy_gui_help (GTK_WINDOW (editor),
-		       "epiphany", 
-		       "ephy-managing-history");
+	ephy_gui_help (GTK_WIDGET (editor), "ephy-managing-history");
 }
 
 static void
-set_columns_visibility (EphyHistoryWindow *view, int value)
+set_column_visibility (EphyHistoryWindow *view,
+		       const char *action_name,
+		       gboolean active)
 {
-	switch (value)
+	if (strcmp (action_name, "ViewTitle") == 0)
 	{
-		case VIEW_TITLE:
-			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, FALSE);
-			break;
-		case VIEW_ADDRESS:
-			gtk_tree_view_column_set_visible (view->priv->title_col, FALSE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
-			break;
+		gtk_tree_view_column_set_visible (view->priv->title_col, active);
+	}
+	if (strcmp (action_name, "ViewAddress") == 0)
+	{
+		gtk_tree_view_column_set_visible (view->priv->address_col, active);
+	}
+	if (strcmp (action_name, "ViewDateTime") == 0)
+	{
+		gtk_tree_view_column_set_visible (view->priv->datetime_col, active);
 	}
 }
 
+static void
+set_all_columns_visibility (EphyHistoryWindow *view,
+			    EphyHistoryWindowColumns details_value)
+{
+	GtkActionGroup *action_group;
+	GtkAction *action_title, *action_address, *action_datetime;
+
+	action_group = view->priv->action_group;
+	action_title = gtk_action_group_get_action (action_group, "ViewTitle");
+	action_address = gtk_action_group_get_action (action_group, "ViewAddress");
+	action_datetime = gtk_action_group_get_action (action_group, "ViewDateTime");
+	
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_title), (details_value & VIEW_TITLE));
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_address), (details_value & VIEW_ADDRESS));
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_datetime), (details_value & VIEW_DATETIME));
+}
+
 static void
 cmd_view_columns (GtkAction *action,
-		  GtkRadioAction *current,
 		  EphyHistoryWindow *view)
 {
-	int value;
+	gboolean active;
+	const char *action_name;
 	GSList *svalues = NULL;
 
-	value = gtk_radio_action_get_current_value (current);
-	set_columns_visibility (view, value);
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	action_name = gtk_action_get_name (action);
+	set_column_visibility (view, action_name, active);
+	
+	svalues = eel_gconf_get_string_list (CONF_HISTORY_VIEW_DETAILS);
 
-	switch (value)
+	if (active)
 	{
-		case VIEW_TITLE:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			svalues = g_slist_append (svalues, (gpointer)"address");
-			break;
+		if (!g_slist_find_custom (svalues, (gpointer) action_name, (GCompareFunc) strcmp))
+		{
+			svalues = g_slist_append (svalues, (gpointer) action_name);
+		}
+	}
+	else
+	{
+		GSList *delete;
+		delete = g_slist_find_custom (svalues, (gpointer) action_name, (GCompareFunc) strcmp);
+		if (delete)
+		{
+			svalues = g_slist_delete_link (svalues, delete);
+		}
 	}
 
 	eel_gconf_set_string_list (CONF_HISTORY_VIEW_DETAILS, svalues);
 	g_slist_free (svalues);
 }
 
-GType
-ephy_history_window_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyHistoryWindowClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_history_window_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyHistoryWindow),
-			0,
-			(GInstanceInitFunc) ephy_history_window_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_WINDOW,
-					       "EphyHistoryWindow",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyHistoryWindow, ephy_history_window, GTK_TYPE_WINDOW)
 
 static void
 ephy_history_window_show (GtkWidget *widget)
@@ -592,7 +573,7 @@ ephy_history_window_show (GtkWidget *widget)
 
 	gtk_widget_grab_focus (window->priv->search_entry);
 
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	GTK_WIDGET_CLASS (ephy_history_window_parent_class)->show (widget);
 }
 
 static void
@@ -601,8 +582,6 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_history_window_finalize;
 
 	object_class->set_property = ephy_history_window_set_property;
@@ -641,7 +620,7 @@ ephy_history_window_finalize (GObject *object)
                          (gpointer *)window);
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_history_window_parent_class)->finalize (object);
 }
 
 static void
@@ -659,6 +638,25 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
+static void
+ephy_history_window_node_middle_clicked_cb (GtkWidget *view,
+					    EphyNode *node,
+					    EphyHistoryWindow *editor)
+{
+	EphyWindow *window;
+	const char *location;
+
+	window = EPHY_WINDOW (get_target_window (editor));
+
+	location = ephy_node_get_property_string
+		(node, EPHY_NODE_PAGE_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+}
+
 static void
 ephy_history_window_update_menu (EphyHistoryWindow *editor)
 {
@@ -666,6 +664,7 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	gboolean cut, copy, paste, select_all;
 	gboolean pages_focus, pages_selection, single_page_selected;
 	gboolean delete, bookmark_page;
+	gboolean bookmarks_locked;
 	int num_pages_selected;
 	GtkActionGroup *action_group;
 	GtkAction *action;
@@ -720,7 +719,8 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	open_in_window = (pages_focus && pages_selection);
 	open_in_tab = (pages_focus && pages_selection);
         delete = (pages_focus && pages_selection);
-	bookmark_page = (pages_focus && single_page_selected);
+	bookmarks_locked = eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING);
+	bookmark_page = (pages_focus && single_page_selected) && !bookmarks_locked;
 
 	action_group = editor->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "OpenInWindow");
@@ -742,6 +742,7 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	gtk_action_set_sensitive (action, delete);
 	action = gtk_action_group_get_action (action_group, "BookmarkLink");
 	gtk_action_set_sensitive (action, bookmark_page);
+
 }
 
 static void
@@ -974,7 +975,13 @@ site_node_selected_cb (EphyNodeView *view,
 	}
 	else
 	{
+		g_signal_handlers_block_by_func (EPHY_SEARCH_ENTRY (editor->priv->search_entry),
+						 G_CALLBACK (search_entry_search_cb),
+						 editor);
 		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
+		g_signal_handlers_unblock_by_func (EPHY_SEARCH_ENTRY (editor->priv->search_entry),
+						   G_CALLBACK (search_entry_search_cb),
+						   editor);
 		setup_filters (editor, TRUE, FALSE);
 	}
 }
@@ -1022,7 +1029,8 @@ build_search_box (EphyHistoryWindow *editor)
 	add_focus_monitor (editor, entry);
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
-	gtk_widget_show (entry);
+    
+	gtk_widget_show_all (entry);
 
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -1132,37 +1140,76 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	g_value_take_object (value, pixbuf);
 }
 
+static void
+convert_cell_data_func (GtkTreeViewColumn *column,
+			GtkCellRenderer *renderer,
+			GtkTreeModel *model,
+			GtkTreeIter *iter,
+			gpointer user_data)
+{
+	int col_id = GPOINTER_TO_INT (user_data);
+	int value;
+	time_t time;
+	char *friendly;
+	
+	gtk_tree_model_get (model, iter,
+			    col_id,	
+			    &value,
+			    -1);
+	time = (time_t) value;
+	
+	friendly = ephy_time_helpers_utf_friendly_time (time);
+	g_object_set (renderer, "text", friendly, NULL);
+	g_free (friendly);
+}
+
+static void
+parse_time_into_date (GtkTreeViewColumn *column,
+		      int column_id)
+{
+	GList *renderers_list;
+	GtkCellRenderer *renderer;
+	
+	renderers_list = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
+	renderer = GTK_CELL_RENDERER (renderers_list->data);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+						 (GtkTreeCellDataFunc) convert_cell_data_func,
+						 GINT_TO_POINTER (column_id),
+						 NULL);
+	g_list_free (renderers_list);
+}
 static void
 view_selection_changed_cb (GtkWidget *view, EphyHistoryWindow *editor)
 {
 	ephy_history_window_update_menu (editor);
 }
 
-static int
+static EphyHistoryWindowColumns
 get_details_value (void)
 {
-	int value;
+	guint value = 0;
 	GSList *svalues;
 
 	svalues = eel_gconf_get_string_list (CONF_HISTORY_VIEW_DETAILS);
-	if (svalues == NULL) return VIEW_TITLE_AND_ADDRESS;
+	if (svalues == NULL) 
+	{
+		svalues = g_slist_append (svalues, (gpointer) "ViewAddress");
+		svalues = g_slist_append (svalues, (gpointer) "ViewTitle");
+		eel_gconf_set_string_list (CONF_HISTORY_VIEW_DETAILS, svalues);
+		return (VIEW_ADDRESS | VIEW_TITLE);
+	}
 
-	if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp) &&
-	    g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewTitle", (GCompareFunc)strcmp))
 	{
-		value = VIEW_TITLE_AND_ADDRESS;
+		value |= VIEW_TITLE;
 	}
-	else if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewAddress", (GCompareFunc)strcmp))
 	{
-		value = VIEW_TITLE;
+		value |= VIEW_ADDRESS;
 	}
-	else if (g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	if (g_slist_find_custom (svalues, "ViewDateTime", (GCompareFunc)strcmp))
 	{
-		value = VIEW_ADDRESS;
-	}
-	else
-	{
-		value = VIEW_TITLE_AND_ADDRESS;
+		value |= VIEW_DATETIME;
 	}
 
 	g_slist_foreach (svalues, (GFunc) g_free, NULL);
@@ -1182,7 +1229,8 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	EphyNode *node;
 	GtkUIManager *ui_merge;
 	GtkActionGroup *action_group;
-	int url_col_id, title_col_id, details_value;
+	int url_col_id, title_col_id, datetime_col_id;
+	EphyHistoryWindowColumns details_value;
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (editor));
 
@@ -1206,12 +1254,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 				      G_N_ELEMENTS (ephy_history_ui_entries), editor);
 
 	details_value = get_details_value ();
-	gtk_action_group_add_radio_actions (action_group,
-					    ephy_history_radio_entries,
-					    G_N_ELEMENTS (ephy_history_radio_entries),
-					    details_value,
-					    G_CALLBACK (cmd_view_columns), 
-					    editor);
+	gtk_action_group_add_toggle_actions (action_group,
+					     ephy_history_toggle_entries,
+					     G_N_ELEMENTS (ephy_history_toggle_entries),
+					     editor);
 
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
@@ -1239,7 +1285,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 					"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
 					"shadow_type", GTK_SHADOW_IN,
 					NULL);
-	gtk_paned_add1 (GTK_PANED (hpaned), scrolled_window);
+	gtk_paned_pack1 (GTK_PANED (hpaned), scrolled_window, TRUE, FALSE);
 	gtk_widget_show (scrolled_window);
 	editor->priv->sites_filter = ephy_node_filter_new ();
 	sites_view = ephy_node_view_new (node, editor->priv->sites_filter);
@@ -1288,7 +1334,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  editor);
 
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_paned_add2 (GTK_PANED (hpaned), vbox);
+	gtk_paned_pack2 (GTK_PANED (hpaned), vbox, TRUE, FALSE);
 	gtk_widget_show (vbox);
 
 	gtk_box_pack_start (GTK_BOX (vbox),
@@ -1314,14 +1360,26 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	title_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Title"),
 				                  G_TYPE_STRING, EPHY_NODE_PAGE_PROP_TITLE,
 				                  EPHY_NODE_VIEW_SORTABLE |
-					          EPHY_NODE_VIEW_SEARCHABLE, NULL, &col);
-	gtk_tree_view_column_set_max_width (col, 250);
+					          EPHY_NODE_VIEW_SEARCHABLE |
+						  EPHY_NODE_VIEW_ELLIPSIZED, NULL, &col);
+	gtk_tree_view_column_set_min_width (col, 300);
+	gtk_tree_view_column_set_resizable (col, TRUE);
 	editor->priv->title_col = col;
+	
 	url_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				                G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
-				                EPHY_NODE_VIEW_SORTABLE, NULL, &col);
-	gtk_tree_view_column_set_max_width (col, 200);
+				                EPHY_NODE_VIEW_SORTABLE |
+						EPHY_NODE_VIEW_ELLIPSIZED, NULL, &col);
+	gtk_tree_view_column_set_min_width (col, 300);
+	gtk_tree_view_column_set_resizable (col, TRUE);
 	editor->priv->address_col = col;
+
+	datetime_col_id = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Date"),
+						     G_TYPE_INT, EPHY_NODE_PAGE_PROP_LAST_VISIT,
+						     EPHY_NODE_VIEW_SORTABLE, NULL, &col);
+	editor->priv->datetime_col = col;
+	parse_time_into_date (editor->priv->datetime_col, datetime_col_id);
+
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (pages_view),
 					   page_drag_types,
 				           G_N_ELEMENTS (page_drag_types),
@@ -1337,6 +1395,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  "node_activated",
 			  G_CALLBACK (ephy_history_window_node_activated_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (pages_view),
+			  "node-middle-clicked",
+			  G_CALLBACK (ephy_history_window_node_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (pages_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_history_window_show_popup_cb),
@@ -1358,7 +1420,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			       "history_paned",
 		               130);
 
-	set_columns_visibility (editor, details_value);
+	set_all_columns_visibility (editor, details_value);
 	setup_filters (editor, TRUE, TRUE);
 }
 
@@ -1497,5 +1559,5 @@ ephy_history_window_dispose (GObject *object)
 		save_date_filter (editor);
 	}
 
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (ephy_history_window_parent_class)->dispose (object);
 }
